#include "StdAfx.h"

#include "RingElemRep.h"
#include "IntegerRep.h"
#include "PolynomialRep.h"

#include "VarList.h"
#include "Ring.h"
#include "RingRep.h"

#include "RingMap.h"
#include "RingMapRep.h"

#include "RingElem.h"

#include "Monomial.h"
#include "MonomialOrder.h"


using namespace std;


Term & Term::multM (const Term & termB) {

	cf = cf->mult(termB.cf);
	mon.multM(termB.mon);

	return *this;

}

Term & Term::addM (const Term & termB) {

	cf = cf->add(termB.cf);
	return *this;

}

ostream& operator<< (ostream& os, const Term & term) {

	os << "( ";
	os << term.cf;
	os << " , ";
	os << term.mon;
	os << " )";

	return os;
}

int PolynomialRep::instanceCount = 0;


PolynomialRep::PolynomialRep(void): theRing(PolynomialRing::integerRing)
{
	instanceCount++;
}


PolynomialRep::~PolynomialRep(void)
{
	cout << "~PolynomialRep: instanceCount = " << instanceCount << endl;
	instanceCount--;
}

PolynomialRep::PolynomialRep(const Integer &rhs):theRing(PolynomialRing::integerRing) {

	Monomial mon((theRing->getVarList()).size());

	Term t(rhs,mon);

	termlist.push_back(t);

	instanceCount++;

}

PolynomialRep::PolynomialRep(const PolynomialRing& pr, const Symbol& x): theRing(pr) {

	Ring cfr = pr->getCoefRing();
	RingElem u = cfr->unity();

	Monomial mon = (pr->getVarList()).makeMonomial(x);

	Term t(u,mon);

	termlist.push_back(t);

	instanceCount++;

}

PolynomialRep::PolynomialRep(const PolynomialRing& pr): theRing(pr) {

	Monomial mon((pr->getVarList()).size());

	Term t(pr->getCoefRing()->unity(), mon);

	termlist.push_back(t);

	instanceCount++;

}

PolynomialRep::PolynomialRep(const PolynomialRing& pr, const Integer val): theRing(pr) {

	Monomial mon((pr->getVarList()).size());

	Term t(val, mon);

	termlist.push_back(t);

	instanceCount++;

}


RingElem PolynomialRep::negate() {
	return Polynomial(this);
}

//
// add and mult don't modify the this-Object
//


RingElem PolynomialRep::add(const RingElem& b) {
	return b->add(Polynomial(this)); 
}


RingElem PolynomialRep::mult(const RingElem& b) {
	return b->mult(Polynomial(this));
}


RingElem PolynomialRep::add(const Integer& b) {
	return Polynomial(this)->add(Polynomial(b));
}



RingElem PolynomialRep::mult(const Integer& b) {
	return Polynomial(this)->mult(Polynomial(b));
}


Polynomial PolynomialRep::merge_add (const Polynomial & a, const Polynomial & b ) {

	return a;

}

Polynomial PolynomialRep::merge_add (const Polynomial & a ) {

	TermListType & tLis = termlist;
	TermListType aLis = a->getTermList();

	TLTI aktTermT = tLis.begin();
	TLTI aktTermA = aLis.begin();

	TLTI aktTermTEnd = tLis.end();
	TLTI aktTermAEnd = aLis.end();

	MonomialOrder ord = theRing->getMonomialOrder();

	while (1) {
	
		if (aktTermT == aktTermTEnd)
			break;
		if (aktTermA == aktTermAEnd)
			break;

		MonCompCode cmp = ord.cmp (aktTermT->mon, aktTermA->mon);

		if (cmp == MON_EQUAL) {
			aktTermT->addM(*aktTermA);
			++aktTermT;
			++aktTermA;
		} else if (cmp == MON_GREATER) {
			++aktTermT;
		} else if (cmp == MON_LESS) {
			aktTermT = tLis.insert(aktTermT,*aktTermA);
			++aktTermT;
			++aktTermA;
		}
	} // while(1)



	if (aktTermT == aktTermTEnd) {
	
		while (aktTermA != aktTermAEnd) {
			if (static_cast<IntegerRep *>(aktTermA->cf.getRep())->getVal() == 0) {
				++aktTermA;
			} else {
				aktTermT = tLis.insert(aktTermT, *aktTermA);
					++aktTermT;
					++aktTermA;
			}
		}
	}

	// provisorische Elimination von Nullkoeffizienten

	aktTermT = tLis.begin();
	aktTermTEnd = tLis.end();
	while (aktTermT != aktTermTEnd) {
		if (static_cast<IntegerRep *>(aktTermT->cf.getRep())->getVal() == 0) {
			aktTermT = tLis.erase(aktTermT);
		} else {
			++aktTermT;
		}
	}

	return Polynomial(this);

}

Polynomial PolynomialRep::merge_add_multerm (const Polynomial & a, const Term & mulTerm ) {

	TermListType & tLis = termlist;
	TermListType aLis = a->getTermList();

	TLTI aktTermT = tLis.begin();
	TLTI aktTermA = aLis.begin();

	TLTI aktTermTEnd = tLis.end();
	TLTI aktTermAEnd = aLis.end();

	MonomialOrder ord = theRing->getMonomialOrder();

	while (1) {
	
		if (aktTermT == aktTermTEnd)
			break;
		if (aktTermA == aktTermAEnd)
			break;

		Term termA = *aktTermA;
		termA = termA.multM(mulTerm);

		MonCompCode cmp = ord.cmp (aktTermT->mon, termA.mon);

		if (cmp == MON_EQUAL) {
			aktTermT->addM(termA);
			++aktTermT;
			++aktTermA;
		} else if (cmp == MON_GREATER) {
			++aktTermT;
		} else if (cmp == MON_LESS) {
			aktTermT = tLis.insert(aktTermT,termA);
			++aktTermT;
			++aktTermA;
		}
	} // while(1)



	if (aktTermT == aktTermTEnd) {
	
		while (aktTermA != aktTermAEnd) {
			Term termA = *aktTermA;
			termA = termA.multM(mulTerm);
			aktTermT = tLis.insert(aktTermT, termA);
			++aktTermT;
			++aktTermA;
			
		}
	}

	// provisorische Elimination von Nullkoeffizienten

	aktTermT = tLis.begin();
	aktTermTEnd = tLis.end();
	while (aktTermT != aktTermTEnd) {
		if (static_cast<IntegerRep *>(aktTermT->cf.getRep())->getVal() == 0) {
			aktTermT = tLis.erase(aktTermT);
		} else {
			++aktTermT;
		}
	}

	return Polynomial(this);

}


Polynomial PolynomialRep::mul_term (const Polynomial & a, const Term & mulTerm ) {

	//cout << "mul_term: a = " << a << endl;
	//cout << "mul_term: mulTerm = " << mulTerm << endl;

	Polynomial a1 = a->clone();

	TermListType & aLis = a1->getTermList();

	for (TLTI aktTermA = aLis.begin(); aktTermA != aLis.end(); ++aktTermA ) {
	
		aktTermA->multM(mulTerm);
	};

	//cout << "mul_term: Result a = " << a1 << endl; 

	return a1;

}


void PolynomialRep::accum_mul_term (Polynomial & a1, const Polynomial & a, const Term & mulTerm ) {

	//cout << "accum mul term: a1 = " << a1 << endl;
	//cout << "a = " << a << endl;
	//cout << "mulTerm = " << mulTerm << endl;

	a1->merge_add_multerm ( a, mulTerm );

	//cout << "a1 neu = " << a1 << endl; 

}


Polynomial PolynomialRep::do_mult (const Polynomial & a, const Polynomial & b ) {

	//cout << "do mult: a = " << a << endl;
	//cout <<	"do mult: b = " << b << endl;

	TermListType & bLis = b->getTermList();

	Polynomial a1 = Polynomial(new PolynomialRep(theRing, Integer(0)));

	//cout << "zero a1 = " << a1 << endl;

	for (TLTCI aktTermB = bLis.begin(); aktTermB != bLis.end(); ++aktTermB ) {

		accum_mul_term (a1, a, *aktTermB );

	};

	return a1;

}

Polynomial PolynomialRep::do_add (const Polynomial & a, const Polynomial & b ) {

	Polynomial a1 = a->clone ();
	return a1->merge_add (b);

}


RingElem PolynomialRep::add(const Polynomial &b) {

	Polynomial a = Polynomial(this);

	if (theRing == b->theRing) {

		return do_add (a, b);

	} else {
	
		RingProd ringProd = theRing->commonOverRing( b->getRing() );
		Polynomial a1 = ringProd.getMapA()->apply(a);
		Polynomial b1 = ringProd.getMapB()->apply(b);

		return do_add (a1, b1);

	}
}

RingElem PolynomialRep::mult(const Polynomial &b) {
	Polynomial a = Polynomial(this);

	if (theRing == b->theRing) {

		return do_mult(a, b);

	} else {
	
		RingProd ringProd = theRing->commonOverRing( b->getRing() );
		Polynomial a1 = ringProd.getMapA()->apply(a);
		Polynomial b1 = ringProd.getMapB()->apply(b);

		return do_mult (a1, b1);

	}
}

//
// addM and multM modify the this-object
//


RingElem PolynomialRep::addM(const Integer& b)  {
	throw RingElemOpVirtual();
}

RingElem PolynomialRep::addM(const Polynomial& b)  {
	return b;	
}



RingElem PolynomialRep::multM(const Integer& b)  {
	throw RingElemOpVirtual();
}

RingElem PolynomialRep::multM(const Polynomial& b)  {
	return b;
}




RingElem PolynomialRep::applyIntMap(const RingMapInt& f) {
	throw Undefined();
}


RingElem PolynomialRep::applyPolyMap(const RingMapPoly& f) {
	return Polynomial(this);
}


ostream& print(ostream& os, VarList& vl, const Monomial& mon) {

	return mon.print(os, vl);

}

void PolynomialRep::print(ostream& os) const {
	
	bool a, b;

	printRingElement(os, a, b);

}

void printTerm (ostream& os, const Term& t, 
									VarList& vl, bool& minslds, bool isfollower) {

		ostringstream coefOs;
		bool iss;

		(t.cf)->printRingElement(coefOs, iss, minslds);
		bool minsldsout = minslds;

		if ( (!minslds) && isfollower )
				os << "+";

		if (iss) {
			os << "(";
			minsldsout = false;
		};

		os << coefOs.str();

		if (iss)
			os << ")";

		os << "*";

		::print(os,vl,t.mon);
		minslds = minsldsout;
}


void PolynomialRep::printRingElement(ostream& os, bool& issum, bool& minusleads) const {

	VarList vl = theRing->getVarList();

	TLTCI p = termlist.begin();
	TLTCI pe = termlist.end();

	bool empty = (p == pe);

	issum = false;

	if (!empty) {
		printTerm(os, *p, vl, minusleads, false);
		++p;
		
		issum = (p != pe);

		for (p = p; p != pe; ++p) {
			bool minslds;

			printTerm(os, *p, vl, minslds, true);

		};
	} else {
		throw Undefined();
	};

}
