#pragma once

#include <list>
#include <utility>

#include <iostream>
#include <sstream>

#include "RingElemRep.h"

#include "Integer.h"


#include "Polynomial.h"
#include "RingElem.h"

#include "Monomial.h"
#include "MonomialOrder.h"

#include "Ring.h"

#include "VarList.h"

class RingMapInt;
class RingMapPoly;

class PolynomialRing;
class Symbol;

class Term {

	friend std::ostream& operator<< (std::ostream& os, const Term & term);

public:

	Term(const RingElem & acf, const Monomial & amon ): cf(acf), mon(amon) {
	};

	Term & multM (const Term & termB);
	Term & addM (const Term & termB);

	RingElem cf;
	Monomial mon;

};



class PolynomialRep: public RingElemRep
{
public:

	static int instanceCount;

	typedef std::list<Term> TermListType;

	typedef TermListType::const_iterator TLTCI;
	typedef TermListType::iterator TLTI;


	virtual ~PolynomialRep(void);
	PolynomialRep(const Integer &rhs);
	
	PolynomialRep(const PolynomialRing& pr, const Symbol& x);
	PolynomialRep(const PolynomialRing& pr);
	PolynomialRep(const PolynomialRing& pr, const Integer val);


	PolynomialRep(const PolynomialRep & rhs): termlist(rhs.termlist), varlist(rhs.varlist), theRing(rhs.theRing) {
		instanceCount++;
	};


	PolynomialRep* clone () {
		return new PolynomialRep(*this); 
	};


	RingElem negate();
	RingElem add(const RingElem& b);
	RingElem mult(const RingElem& b);

	RingElem add(const Integer& b);
	RingElem mult(const Integer& b);

	RingElem add(const Polynomial& b);
	RingElem mult(const Polynomial& b);




	Polynomial merge_add (const Polynomial & a, const Polynomial & b );
	Polynomial merge_add (const Polynomial & a );
	Polynomial merge_add_multerm (const Polynomial & a, const Term & mulTerm );


	Polynomial mul_term (const Polynomial & a, const Term & mulTerm );
	void accum_mul_term (Polynomial & a1, const Polynomial & a, const Term & mulTerm );
	
	Polynomial do_add (const Polynomial & a, const Polynomial & b );
	Polynomial do_mult (const Polynomial & a, const Polynomial & b );

	virtual RingElem addM(const Integer& b);
	virtual RingElem addM(const Polynomial& b);

	virtual RingElem multM(const Integer& b);
	virtual RingElem multM(const Polynomial& b);


	RingElem applyIntMap(const RingMapInt& f);
	RingElem applyPolyMap(const RingMapPoly& f);

	virtual void print(std::ostream& os) const;
	virtual void printRingElement(std::ostream& os, bool& issum, bool& minusleads) const;

	PolynomialRing & getRing() {
		return theRing;
	};

	TermListType & getTermList () {
		return termlist;
	};

	VarList & getVarList () {
		return varlist;
	}

protected:

	PolynomialRep();


private:

	PolynomialRing theRing;

	VarList varlist;

	TermListType termlist;

};

std::ostream& print(std::ostream& os, VarList& vl, Monomial& mon);

