#pragma once


#include "Ring.h"

#include "RingElem.h"
#include "Polynomial.h"

#include "RingProd.h"

#include "VarList.h"
#include "MonomialOrder.h"



class RingRep: public ExprRep {

public:

	RingRep() {};
	~RingRep() {};

	virtual RingProd commonOverRing (const Ring& rB) const { return RingProd (); };
	virtual RingProd commonOverRingPolyLeft (const PolynomialRing& rB) const { return RingProd(); };

	virtual RingElem unity ();

	virtual void print (std::ostream &os) const {
		os << "Ring";
	};

	virtual bool eq (const Ring& rB) const;
	virtual bool eq (const IntegerRing& rB) const;
	virtual bool eq (const PolynomialRing& rB) const;


};


class IntegerRingRep: public RingRep {

public:

	IntegerRingRep() {};
	~IntegerRingRep() {};

	virtual void print (std::ostream &os) const {
		os << "ZZ";
	};

	RingProd commonOverRing (const Ring& rB) const { 
		return RingProd(IntegerRing::integerRing, rB); };

	RingProd commonOverRingPolyLeft (const PolynomialRing& rB) const { 
		return RingProd(rB, IntegerRing::integerRing); };

	virtual RingElem unity ();

	virtual bool eq (const Ring& rB) const;
	virtual bool eq (const IntegerRing& rB) const;
	virtual bool eq (const PolynomialRing& rB) const;


};

class PolynomialRingRep: public RingRep {

public:

	PolynomialRingRep(const Ring& acoefRing): coefRing(acoefRing),varlist(VarList()) {
	};

	PolynomialRingRep(const VarList& avarlist):
			coefRing(IntegerRing::integerRing), varlist(avarlist) {
	};

	PolynomialRingRep(const Ring& acoefRing, const VarList& avarlist):
		coefRing(acoefRing), varlist(avarlist) {
	};

	~PolynomialRingRep();

	VarList getVarList () const { return varlist; };
	Ring getCoefRing () const { return coefRing; };
	MonomialOrder getMonomialOrder () const { return monOrder; };
   

	Polynomial makePolynomial(const Symbol& x);

	RingProd commonOverRing (const Ring& rB) const;
	RingProd commonOverRingPolyLeft (const PolynomialRing& rB) const;


	virtual void print(std::ostream &os) const;

	virtual RingElem unity ();

	virtual bool eq (const Ring& rB) const;
	virtual bool eq (const IntegerRing& rB) const;
	virtual bool eq (const PolynomialRing& rB) const;


	protected:	

	PolynomialRingRep ();

	private:


		Ring coefRing;
		VarList varlist;
	
		MonomialOrder monOrder;
};


